#include "problem.hh"

#include <fstream.h>
#include <stdlib.h>
#include <math.h>



/****************************************************************************/
/* Solution                                                                 */
/****************************************************************************/


SA_Solution::SA_Solution () 
{
	n = 0;
	phi = Array<int>(0);
	i = j = -1;
	oldcost = newcost = -1;
}


void SA_Solution::Copy (SA_Solution& source) 
{ 
	*this = source;
}


ostream& operator<< (ostream &out, SA_Solution& sol) 
{
	out << "n= " << sol.n << endl;
	out << "cost= " << sol.newcost << endl;
//	out << "phi= " << sol.phi << endl;
	return out; 
}






/****************************************************************************/
/* Problem                                                                  */
/****************************************************************************/


SA_Problem::SA_Problem () 
{ 
	n = 0;
	m = 0;
}


SA_Problem::~SA_Problem  ()
{
}


void SA_Problem::EvaluateOptions (int* argc,char*** argv) 
{
}


int SA_Problem::ReadProblemData (char *filename)
{   
  	ifstream f(filename);
  	f >> n;
  	f >> m;
  	deg = Array<int>(n);
  	edg = Array<int>(2*m+1);
  	adj = Array<int>(n+1);
	f >> deg;
	f >> edg;
	f >> adj;
	
	// Sort the edges, just in case
	for (int i=0; i<n; i++) {
		edg.sort(adj[i],adj[i+1]-1);
	}

  	if (n <= 30) {
    	globaln = 1;
    	for (int k = 1; k <= n; k++) {
	      	globaln *= globaln;
    	}
	} else {
    	globaln = 1E30;
  	}

	localn = (float(n)*(float(n)-1)) / 2;


	return 1;
}


	
SA_Solution* SA_Problem::CreateSolution ()
{ 
   	SA_Solution* newsol = new SA_Solution;   
   	newsol->n = n;
   	newsol->phi = Array<int>(n);
   	return newsol;
}


void SA_Problem::GetInitialSolution (SA_Solution& sol)
{ 
  	for (int u=0; u<n; u++) {
    	sol.phi[u] = u;
  	}
	
	sol.i = sol.j = -1;
  	
	int c = 0;
	for (int u=0; u<n; u++) {
		for (int i=adj[u]; i<adj[u+1]; i++) {
			const int v = edg[i];
			if (v>u) break;
			c += abs(sol.phi[u]-sol.phi[v]);
	}	}
  
  	sol.newcost = sol.oldcost = c;
}


void SA_Problem::GetRandomSolution (SA_Solution& sol)
{
  	for (int u=0; u<n; u++) {
    	sol.phi[u] = u;
  	}
	sol.phi.permute();
	
	sol.i = sol.j = -1;
  	
	int c = 0;
	for (int u=0; u<n; u++) {
		for (int i=adj[u]; i<adj[u+1]; i++) {
			const int v = edg[i];
			if (v>u) break;
			c += abs(sol.phi[u]-sol.phi[v]);
	}	}
  
  	sol.newcost = sol.oldcost = c;
}



float SA_Problem::GetCost (SA_Solution& sol)
{
	return sol.newcost;
}



inline int cost (const SA_Problem& pbm, const SA_Solution& sol) {
	int c = 0;
	for (int u=0; u<pbm.n; u++) {
		for (int i=pbm.adj[u]; i<pbm.adj[u+1]; i++) {
			const int v = pbm.edg[i];
			if (v>u) break;
			c += abs(sol.phi[u]-sol.phi[v]);
	}	}
	return c;
}


void SA_Problem::GetNeighbor (SA_Solution& sol, float rel_temp)
{
	sol.i = MyRandom(n);
	sol.j = MyRandom(n);
	
	if (sol.i==sol.j) return;	
	
	int d=0;
	for (int i=adj[sol.i]; i<adj[sol.i+1]; i++) d-=abs(sol.phi[sol.i]-sol.phi[edg[i]]);
	for (int i=adj[sol.j]; i<adj[sol.j+1]; i++) d-=abs(sol.phi[sol.j]-sol.phi[edg[i]]);
	sol.phi.swap(sol.i,sol.j);
	for (int i=adj[sol.i]; i<adj[sol.i+1]; i++) d+=abs(sol.phi[sol.i]-sol.phi[edg[i]]);
	for (int i=adj[sol.j]; i<adj[sol.j+1]; i++) d+=abs(sol.phi[sol.j]-sol.phi[edg[i]]);

   	sol.newcost = sol.newcost+d;
	if (sol.newcost<=0 || sol.newcost<=0) {
		cerr << "cagada" << endl;
		abort();
	}

	static int count=0;
	if (count++==n*50) {
		cerr << "cost: " << sol.oldcost << endl << flush;
		count=0;
	}
}




void SA_Problem::ResetSolution (SA_Solution& sol)
{
	sol.phi.swap(sol.i,sol.j);
  	sol.j = sol.i = -1;
  	sol.newcost = sol.oldcost;
}


void SA_Problem::UpdateSolution (SA_Solution& sol)
{
  	sol.i = sol.j = -1;
  	sol.oldcost = sol.newcost;
}


float SA_Problem::GetLocalN ()
{ 
	return localn;
}


float SA_Problem::GetGlobalN ()
{ 
	return globaln;
}


void SA_Problem::Copy (SA_Solution& source, SA_Solution& dest)
{
	dest = source;
}	


void SA_Problem::OutputSolution (ostream& out, SA_Solution& sol)
{ 
	out << sol;
}


int SA_Problem::InputSolution (istream& in, SA_Solution& sol)
{ 
	in >> sol.n;
	in >> sol.newcost;
	sol.phi = Array<int>(sol.n);
	in >> sol.phi;
	return 1;
}


SA_Move* SA_Problem::CreateMove ()
{
}


int	SA_Problem::ExtractMove(SA_Solution& sol, SA_Move& m)
{
}


int SA_Problem::ApplyMove(SA_Solution& sol, SA_Move& m)	
{
}


void SA_Problem::OutputMove (ostream& out, SA_Move& m)
{
}


int SA_Problem::InputApplyMoves(istream& in, SA_Solution& sol)												
{
}


int SA_Problem::OutputSolution(char* filename, SA_Solution& sol)
{
	ofstream f(filename);
	f << sol;
	return 1;
}

	
int SA_Problem::InputSolution (char* filename, SA_Solution& sol)
{
	cerr << __PRETTY_FUNCTION__ << cerr;
	abort();
}
											

void SA_Problem::Postprocessing (SA_Solution& sol)
{
}


int	SA_Problem::Reoptimize (SA_Solution& sol)
{
}






// ===========> CLASS MOVE (optional) <==============

SA_Move::SA_Move ()
{
	cerr << __PRETTY_FUNCTION__ << cerr;
	abort();
}

SA_Move::~SA_Move ()
{
	cerr << __PRETTY_FUNCTION__ << cerr;
	abort();
}

SA_Move::SA_Move (SA_Problem& pbm)
{
	cerr << __PRETTY_FUNCTION__ << cerr;
	abort();
}



