#ifndef Fiedler_hh
#define Fiedler_hh


#include "Array.hh"
#include "Graph.hh"



/**
	@short Class to compute Fiedler vectors of graphs.
	@version 1.0
	@author Jordi Petit i Silvestre <jpetit@lsi.upc.es>
	
	Class to compute Fiedler vectors of graphs.

	This class uses Chaco in order to compute the Fiedler vector of a graph.
	With this information, it also computes the fiedler eigenvalue and the
	fiedler permutation.

*/


class Fiedler 
{
	private:

		/** The binded graph. */
		const Graph* _G;
		/** The fiedler eigenvalue. */
		double _eig;
		/** The fiedler eigenvector. */
		Array<double> _vec;
		/** The fiedler permutation. */
		Array<int> _ord;		

	public:

		/** Constructs a Fiedler computation. The input graph must be connected and non empty. */
		Fiedler (const Graph& graph);

		/** Executes the Fiedler computation. */
		Fiedler& operator() ();

		/** Returns the fiedler eigenvalue. */
		double eigenvalue () const {return _eig;}
		/** Returns the fiedler eigenvector. */
		const Array<double>& eigenvector () const {return _vec;}
		/** Returns the i-th component of the fiedler eigen vector. */
		double eigenvector (const int i) const {return _vec[i];}
		/** Returns the fiedler permutation. */
		const Array<int>& permutation () const {return _ord;}
		/** Returns the position of node u in the fiedler permutation. */
		int permutation (const int u) const {return _ord[u];}

		/** Returns the binded graph. */
		const Graph& graph () const {return *_G;}

};


#endif

