#include "Graph.hh"
#include <assert.h>





// ************************************************************************************ 
// Constructors
// ************************************************************************************ 


Graph::Graph (const string& f) 
{
	gen_empty();
	if (f!="") {
		ifstream ifs(f);
		get_gra(ifs);
}	}




// ************************************************************************************ 
// Input/Output
// ************************************************************************************ 


void Graph::get_gra (const string& filename) 
{
	ifstream ifs(filename);
	get_gra(ifs);
}


void Graph::put_gra (const string& filename) const
{
	ofstream ofs(filename);
	put_gra(ofs);
}


void Graph::put_cha (const string& filename) const
{
	ofstream ofs(filename);
	put_cha(ofs);
}


void Graph::put_rmf (const string& filename) const
{
	ofstream ofs(filename);
	put_rmf(ofs);
}




void Graph::get_gra (istream& is)
{
	is >> _n;
	is >> _m;	
	_deg=Array<int>(_n);
	_edg=Array<int>(2*_m+1);
	_adj=Array<int>(_n+1);
	is >> _deg;
	is >> _edg;
	is >> _adj;
	
	for (int i=0; i<_n; i++) {
		_edg.sort(_adj[i],_adj[i+1]-1);
	}
}


void Graph::put_gra (ostream& os) const
{
	os << _n;
	os << _m;	
	os << _deg;
	os << _edg;
	os << _adj;
}


void Graph::put_cha (ostream& os) const
{
	os << nodes() << " " << edges() << endl;
	for (int u=0; u<nodes(); u++) {
		for (int i=adj(u); i<adj(u+1); i++) {
			os << edg(i)+1 << " ";
		}
		os << endl;
}	}


void Graph::put_rmf (ostream& os) const
{
	os << "p ghct " << nodes() << " " << edges() << endl;
	for (int u=0; u<nodes(); u++) {
		for (int i=adj(u); i<adj(u+1); i++) {
			if (u<edg(i)) {
				os << "e " << u+1 << " " << edg(i)+1 << " 1" << endl;
}	}	}	}



// ************************************************************************************ 
// Utilities
// ************************************************************************************ 


bool Graph::connected (const int u, const int v) const
{
	if (v==u) return false;

	for (int i=adj(u); i<adj(u+1); i++) {
		if (edg(i)==v) return true;
	}	
	return false;
}



void Graph::gomory_hu_tree (UGRAPH<int,int>& T) const
{
	extern void gomory_hu (int n, int m, int *u, int *v, int *uu, int *vv, int *ff);

	int u[edges()];
	int v[edges()];
	int uu[nodes()-1];
	int vv[nodes()-1];
	int ff[nodes()-1];
	
	for (int i=0,c=0; i<nodes(); i++) {
		for (int j=adj(i); j<adj(i+1); j++) {
			const int k=edg(j);
			if (i<k) {
				u[c]=i;
				v[c]=k;
				c++;
	}	}	}
	
	gomory_hu(nodes(),edges(),u,v,uu,vv,ff);

	T.clear();
	node V[nodes()];
	for (int i=0; i<nodes(); i++) {
		V[i]=T.new_node(i);	
	}
	for (int i=0; i<nodes()-1; i++) {
		T.new_edge(V[uu[i]],V[vv[i]],ff[i]);
}	}



void Graph::dfs (const int u, Array<bool>& vis) const
{
	vis[u]=true;
	for (int i=adj(u); i<adj(u+1); i++) {
		const int v=edg(i);
		if (!vis[v]) {
			dfs(v,vis);
}	}	}



int Graph::max_degree () const
{
	if (nodes()==0) {
		return -1;
	} else {
		int maxdeg=deg(0);
		for (int u=1; u<nodes(); u++) {
			maxdeg >?= deg(u);
		}
		return maxdeg;
}	}



int Graph::components () const
{
	Array<bool> vis(nodes());
	for (int u=0; u<nodes(); u++) {
		vis[u]=false;
	}
	int c=0;
	for (int u=0; u<nodes(); u++) {
		if (!vis[u]) {
			c++;
			dfs(u,vis);
	}	}
	return c;
}



// ************************************************************************************ 
// Generation 
// ************************************************************************************ 


void Graph::gen_empty ()
{
	_n=0;
	_m=0; 
	_adj=Array<int>(1); 
	_edg=Array<int>(1); 
	_deg=Array<int>(0);
	adj(0)=0;
	edg(0)=-1;
}


void Graph::gen_sample ()
{
	ugraph G;	
	node V[5];
	V[0]=G.new_node(); V[1]=G.new_node(); V[2]=G.new_node(); V[3]=G.new_node(); V[4]=G.new_node();	
	G.new_edge(V[0],V[1]); G.new_edge(V[0],V[3]); G.new_edge(V[0],V[4]); G.new_edge(V[1],V[2]);
	G.new_edge(V[1],V[4]); G.new_edge(V[2],V[3]); G.new_edge(V[2],V[4]); G.new_edge(V[4],V[3]);
	gen_ugraph(G);
}


void Graph::gen_ugraph (const ugraph& G)
{
	_n=G.number_of_nodes();
	_m=G.number_of_edges();
	_adj=Array<int>(_n+1);
	_deg=Array<int>(_n);
	_edg=Array<int>(2*_m+1);
	
	int i,j;
	node u,v;
	node_array<int> A(G);
	
	i=0;
	forall_nodes(v,G) {
		A[v]=i++;
	}
	i=j=0; adj(0)=0;
	forall_nodes(v,G) {
		deg(i)=0;
		forall_adj_nodes(u,v) {
			deg(i)++;
			edg(j++)=A[u];
		}
		adj(i+1)=adj(i)+deg(i);
		_edg.sort(adj(i),adj(i)+deg(i)-1);
		i++;
	}
	edg(j)=-1;
}


void Graph::gen_random_np (const int n, const double p)
{
	ugraph G;
	Array<node> V(n);
	for (int i=0; i<n; i++) {
		V[i]=G.new_node();
	}
	for (int i=0; i<n; i++) {
		for (int j=i+1; j<n; j++) {
			if (Random(p)) {	
				G.new_edge(V[i],V[j]);
	}	}	}	
	gen_ugraph(G);
}


void Graph::gen_random_nm (const int n, const int m)
{
	ugraph G;
	random_graph(G,n,m);
	gen_ugraph(G);
}


void Graph::gen_complete (const int n)
{
	ugraph G;
	complete_graph(G,n);
	gen_ugraph(G);
}


void Graph::gen_complete_bipartite (const int a, const int b)
{
	ugraph G;
	list<node> la,lb;
	complete_bigraph(G,a,b,la,lb);
	gen_ugraph(G);
}


void Graph::gen_random_bipartite_m (const int a, const int b, const int m)
{
	ugraph G;
	list<node> la,lb;
	random_bigraph(G,a,b,m,la,lb);
	gen_ugraph(G);
}


void Graph::gen_random_bipartite_p (const int a, const int b, const double p)
{
	ugraph G;
	list<node> la,lb;
	node u,v;

	random_bigraph(G,a,b,0,la,lb);
	forall (u,la) {
		forall (v,la) {
			if (Random(p)) {
				G.new_edge(u,v);
	}	}	}
	gen_ugraph(G);
}


void Graph::gen_grid (const int x)
{
	ugraph G;
	grid_graph(G,x);
	gen_ugraph(G);
}


void Graph::gen_random_linear (const int n, const double c)
{
	ugraph G;
	Array<node> V(n);

	for (int i=0; i<n; i++) {
		V[i]=G.new_node();
	}
	for (int i=0; i<n; i++) {
		for (int j=i+1; j<n; j++) {
			if (Random(c/(j-i+1))) {
				G.new_edge(V[i],V[j]);				
	}	}	}
	
	gen_ugraph(G);
}



void Graph::gen_bond_percolation (const double)
{
	assert(false);
}


void Graph::gen_site_percolation (const double p)
{
	cerr << "No s pas si gen_site_percolation funciona correctament. Cal verificar-ho. " << endl;
	int i,j,n,np,k;
	const int s=500;
	const int maxint = MAXINT;
	
	int m[s][s];
	for (i=0; i<s; i++) {
		for (j=0; j<s; j++) {
			m[i][j]=maxint;
	}	}
		
	n=1;
	m[s/2][s/2]=-n;
	k=0;
	do {
		np=n;

		for (i=s/2-k; i<=s/2+k; i++) {
			for (j=s/2-k; j<=s/2+k; j++) {
				if (m[i][j]<0) {
					m[i][j]=-m[i][j];
					if (m[i-1][j]==maxint && Random(p)) {
						n++; m[i-1][j]=-n; 
					} 
					if (m[i+1][j]==maxint && Random(p)) {
						n++; m[i+1][j]=-n; 
					} 
					if (m[i][j-1]==maxint && Random(p)) {
						n++; m[i][j-1]=-n; 
					} 
					if (m[i][j+1]==maxint && Random(p)) {
						n++; m[i][j+1]=-n; 
					} 
		}	}	}
		k++;
	} while (n!=np);	

	ugraph G;
	node V[n];
	
	for (i=s/2-k; i<=s/2+k; i++) {
		for (j=s/2-k; j<=s/2+k; j++) {
			if (m[i][j]!=maxint) {
				V[m[i][j]-1]=G.new_node();			
	}	}	}
	
	for (i=1; i<s-1; i++) {
		for (j=1; j<s-1; j++) {
			if (m[i][j]!=maxint && m[i-1][j]!=maxint) {
				G.new_edge(V[m[i][j]-1],V[m[i-1][j]-1]);
			}
			if (m[i][j]!=maxint && m[i+1][j]!=maxint) {
				G.new_edge(V[m[i][j]-1],V[m[i+1][j]-1]);			
			}
			if (m[i][j]!=maxint && m[i][j-1]!=maxint) {
				G.new_edge(V[m[i][j]-1],V[m[i][j-1]-1]);			
			}
			if (m[i][j]!=maxint && m[i][j+1]!=maxint) {
				G.new_edge(V[m[i][j]-1],V[m[i][j+1]-1]);			
	}	}	}		
	gen_ugraph(G);
}





