#include "Array.hh"
#include "Graph.hh"
#include "Layout.hh"



/**
	@short Static class that offers different methods to compute lower bounds for the MINLA problem.
	@author Jordi Petit i Silvestre <jpetit@lsi.upc.es>
*/


struct LowerBounds_LA {

	/** Computes a lower bound of the MINLA problem using the Gomory-Hu method based on the min-flow max-cut theorem. */
	static int gomory_hu (const Graph& G);
	/** Computes a lower bound of the MINLA problem using the Juvan-Mohar method based on spectral properties of the graph. */
	static int juvan_mohar (const Graph& G);
	/** Computes a lower bound of the MINLA problem using a method of mine. */
	static int edges_method (const Graph& G);
	/** Computes a lower bound of the MINLA problem using a method of mine. */
	static int degree_method (const Graph& G);
	/** Computes a lower bound of the MINLA problem using a method due to Juvan and Mohar. */
	static int path_method (const Graph& G);
	/** Computes a lower bound of the MINLA problem using a method of mine. */
	static int mesh_method (const Graph& G);
	
};
