#include "Random.hh"



// Define one the following implementations:

// #define RANDOM_STDLIB
#define RANDOM_LEDA





// ***************************************************************************
// Implementation with LEDA
// ***************************************************************************

#ifdef RANDOM_LEDA
#include <LEDA/random_source.h>


void RandomSeed (const int seed) {
	rand_int.set_seed(seed);
}

double Random () {
	double r;
	rand_int >> r;
	return r;  
}

int Random (const int n) {
	return rand_int(0,n-1);
}

int Random (const int low, const int high) {
	return rand_int(low,high);
}

bool Random (const double p) {
	if (p>=1.0) return true;
	if (p<=0.0) return false;
	return Random()<=p;
}


#endif






// ***************************************************************************
// Implementation with STDLIB (rand and srand)
// ***************************************************************************

#ifdef RANDOM_STDLIB
#include <stdlib.h>


void RandomSeed (const int seed) {
	srand(seed);
}

double Random () {
	return rand()/double(RAND_MAX);  
}

int Random (const int n) {
	return rand()%n;
}

int Random (const int low, const int high) {
	return rand()%(high-low+1)+low;
}

bool Random (const double p) {
	if (p>=1.0) return true;
	if (p<=0.0) return false;
	return Random()<=p;
}

#endif





